
# PageMaker.py is part of The Ming Server
# billy_bob_ming@yahoo.com

# Copyright (C) 2005, 2006  Richard Harris, Marco Rimoldi
# Released under the GNU General Public License
# (See the included COPYING file)

# The Ming Server is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# The Ming Server is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with The Ming Server; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
from TheMingServer import Indexer, Logger, consts, lib, exceptions
from TheMingServer.parsers import *
from TheMingServer.makers import *
from TheMingServer.writers import *

## Compression ##
try:
	import zlib
	GZIP = 1
except:
	GZIP = 0

lib.prn(__name__, "init")

class PageMaker:
	NAME = "PageMaker"
	MENU = "menu"
	FOOTER = "footer"
	PARSERS = {
		"html":HTMLParser,
		"plain":PlainParser,
		"pre":PREParser,
		"#python":PythonParser,
		"3tml":TTMLParser,
		}
	MAKERS = {
		"html":HTMLMaker,
		"plain":HTMLMaker,
		"pre":HTMLMaker,
		"#python":HTMLMaker,
		"3tml":HTMLMaker,
		}
	WRITERS = {
		"default":WebWriter,
		"compressed":GZWriter,
		}

	def __init__(self):
		self.source = consts.DATA
		self.files_index = {}
		self.menu = ""
		if consts.LOGGING:
			self.logger = Logger()

	def __index(self, dirname, name):
		id = Indexer()
		self.files_index, self.menu = id.index(dirname, name, self.source)

	def load_package(self, packname):
		lib.prn("page requires the add-on package '%s'" % packname, "INFO")
		exec "from %s import *" % packname in globals()
		for set in ("PARSERS", "MAKERS", "WRITERS"):
			try:
				exec "self.%(set)s['%(type)s'] = %(set)s['%(type)s']" % dict(set=set,
																			 type=self.content_type)
			except:
				pass

	def __read_data(self, fname):
		lines = lib.get_lines(fname)
		if not lines: # we don't have read-access to this file (or it happens to be empty)
			raise exceptions.ForbiddenError
		info = lines[0].strip()
		pieces = info.split(":")
		self.content_type = pieces[-1]
		if len(pieces) > 1:
			self.load_package(packname=pieces[0])
		data = lines[1:]
		lib.prn("page type is '%s'" % self.content_type, "INFO")
		return data

	def __parse_data(self, dirname, name):
		self.fname = dirname + os.sep + name
		data = self.__read_data(self.fname)
		self.title, desc = self.files_index[name]
		parser = self.PARSERS[self.content_type]()
		parsed_data = parser.parse(self.title, desc, data)
		parsed_data[self.MENU] = self.menu
		footer = lib.get_plate("footer.html", dirname)
		parsed_data[self.FOOTER] = footer
		return parsed_data

	def __make_page(self, parsed_data, dirname):
		maker = self.MAKERS[self.content_type](dirname)
		page = maker.make(parsed_data)
		return page

	def __log(self, dirname, name):
		dirname = dirname.replace(self.source, "")
		if dirname:
			dirname += os.sep
			if dirname[0] == os.sep:
				dirname = dirname[1:]
		self.logger.log_entry(self.title, dirname + name)

	def __write(self, page):
		accepted = os.environ.get("HTTP_ACCEPT_ENCODING", "")
		if GZIP and "gzip" in accepted:
			writer = self.WRITERS["compressed"](accepted)
		else:
			writer = self.WRITERS["default"]()
		writer.write(page)

	def page(self, dirname, name):
		self.__index(dirname, name)
		if name not in self.files_index:
			raise exceptions.NotFoundError
		parsed_data = self.__parse_data(dirname, name)
		if parsed_data == None: return 1
		page = self.__make_page(parsed_data, dirname)
		self.__write(page)
		if consts.LOGGING:
			self.__log(dirname, name)

	def prepal(self):
		from TheMingServer.paypal import PrePal
		p = PrePal()
		page = p.decision_page()
		self.__write(page)

	def postpal(self):
		from TheMingServer.paypal import PostPal
		p = PostPal()
		p.deliver()
		self.__write(page)